"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const internals_1 = require("../internals");
const _1 = require(".");
function intersect(other, main) {
    var immutable = core_1.isImmutable(main._owner);
    var outputSet;
    if (Array.isArray(other)) {
        outputSet = internals_1.cloneSortedMap(true, main, true);
        intersectArray(main, outputSet, other);
    }
    else if (core_1.isIterable(other)) {
        if (internals_1.isSortedMap(other)) {
            var it;
            if (_1.size(other) <= _1.size(main)) {
                outputSet = internals_1.cloneSortedMap(true, main, true);
                it = internals_1.iterate(other);
            }
            else {
                outputSet = internals_1.cloneSortedMap(true, other, true);
                it = internals_1.iterate(main);
            }
            intersectSortedMap(main, outputSet, it);
        }
        else {
            outputSet = internals_1.cloneSortedMap(true, main, true);
            intersectIterable(main, outputSet, other[Symbol.iterator]());
        }
    }
    else {
        return main;
    }
    if (_1.size(outputSet) === _1.size(main)) {
        return main;
    }
    if (immutable) {
        return internals_1.refreeze(outputSet);
    }
    main._keyMap = outputSet._keyMap;
    main._sortedValues = outputSet._sortedValues;
    return main;
}
exports.intersect = intersect;
function intersectArray(inputSet, outputSet, array) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    for (var i = 0; i < array.length; i++) {
        var value = array[i];
        if (_1.has(value[0], inputSet)) {
            internals_1.setItem(value[0], value[1], keyMap, sortedValues, select);
        }
    }
}
function intersectIterable(inputSet, outputSet, it) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    var current;
    while (!(current = it.next()).done) {
        var value = current.value;
        if (_1.has(value[0], inputSet)) {
            internals_1.setItem(value[0], value[1], keyMap, sortedValues, select);
        }
    }
}
function intersectSortedMap(inputSet, outputSet, it) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    var current;
    while (!(current = it.next()).done) {
        var entry = current.value;
        if (_1.has(entry.key, inputSet)) {
            internals_1.setItem(entry.key, entry.value, keyMap, sortedValues, select);
        }
    }
}

//# sourceMappingURL=intersect.js.map
