import { isImmutable, isIterable } from '@collectable/core';
import { isSortedMap, refreeze, cloneSortedMap, setItem, iterate } from '../internals';
import { has, size } from '.';
export function intersect(other, main) {
    var immutable = isImmutable(main._owner);
    var outputSet;
    if (Array.isArray(other)) {
        outputSet = cloneSortedMap(true, main, true);
        intersectArray(main, outputSet, other);
    }
    else if (isIterable(other)) {
        if (isSortedMap(other)) {
            var it;
            if (size(other) <= size(main)) {
                outputSet = cloneSortedMap(true, main, true);
                it = iterate(other);
            }
            else {
                outputSet = cloneSortedMap(true, other, true);
                it = iterate(main);
            }
            intersectSortedMap(main, outputSet, it);
        }
        else {
            outputSet = cloneSortedMap(true, main, true);
            intersectIterable(main, outputSet, other[Symbol.iterator]());
        }
    }
    else {
        return main;
    }
    if (size(outputSet) === size(main)) {
        return main;
    }
    if (immutable) {
        return refreeze(outputSet);
    }
    main._keyMap = outputSet._keyMap;
    main._sortedValues = outputSet._sortedValues;
    return main;
}
function intersectArray(inputSet, outputSet, array) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    for (var i = 0; i < array.length; i++) {
        var value = array[i];
        if (has(value[0], inputSet)) {
            setItem(value[0], value[1], keyMap, sortedValues, select);
        }
    }
}
function intersectIterable(inputSet, outputSet, it) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    var current;
    while (!(current = it.next()).done) {
        var value = current.value;
        if (has(value[0], inputSet)) {
            setItem(value[0], value[1], keyMap, sortedValues, select);
        }
    }
}
function intersectSortedMap(inputSet, outputSet, it) {
    var { _keyMap: keyMap, _sortedValues: sortedValues, _select: select } = outputSet;
    var current;
    while (!(current = it.next()).done) {
        var entry = current.value;
        if (has(entry.key, inputSet)) {
            setItem(entry.key, entry.value, keyMap, sortedValues, select);
        }
    }
}

//# sourceMappingURL=intersect.js.map
